<?php

/** Sort by gallery name.        */ define('ANSEL_SORT_NAME', 0);
/** Sort by gallery description. */ define('ANSEL_SORT_DESC', 1);
/** Sort by gallery owner.       */ define('ANSEL_SORT_OWNER', 2);
/** Sort by date-created value.  */ define('ANSEL_SORT_CREATED', 3);
/** Sort by last-modified value. */ define('ANSEL_SORT_MODIFIED', 4);

/**
 * Ansel Base Class.
 *
 * $Horde: ansel/lib/Ansel.php,v 1.94 2004/12/30 15:07:59 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.94 $
 * @package Ansel
 */
class Ansel {

    /**
     * Retrieves the current user's gallery list from storage.  This
     * function will also sort the resulting list, if requested.
     *
     * @param object Ansel_Gallery $storage   The current storage object.
     * @param integer $perm        (optional) The level of permissions to
     *                             require for a gallery to return it.
     * @param string  $owner       (optional) Only return gelleries that the this user owns.
     * @param string  $parent      (optional) The parent share to start searching at.
     * @param boolean $allLevels   (optional) Return all levels, or just the direct children
     *                             of $parent? Defaults to all levels.
     * @param integer $from        The gallery to start listing at.
     * @param integer $count       The number of galleries to return.
     *
     * @return array  Returns a list of matching, sorted galleries.
     */
    function listGalleries($perm = PERMS_SHOW,
                           $owner = null,
                           $parent = DATATREE_ROOT,
                           $allLevels = true,
                           $from = 0,
                           $count = 0)
    {
        global $ansel_shares, $prefs, $notification;

        $groupby = $prefs->getValue('groupby');
        $sortby = $prefs->getValue('sortby');
        $sortdown = $prefs->getValue('sortdir');

        /* Sorting criteria for the gallery list. */
        $sort = array(
            ANSEL_SORT_NAME => 'name',
            ANSEL_SORT_DESC => 'desc',
            ANSEL_SORT_OWNER => 'owner',
            ANSEL_SORT_CREATED => 'date-created',
            ANSEL_SORT_MODIFIED => 'last-modified',
        );

        $galleries = $ansel_shares->listShares(Auth::getAuth(), $perm, $owner, $parent, $allLevels, $from, $count);
        if (is_a($galleries, 'PEAR_Error')) {
            $notification->push(sprintf(_("An error occurred listing galleries: %s"), $galleries->getMessage()), 'horde.error');
            return array();
        }

        /* Return empty lists now. */
        if (!count($galleries)) {
            return $galleries;
        }

        $gallery_groups = array();

        switch ($groupby) {
        case 'owner':
            require_once 'Horde/Identity.php';

            /* Put galleries in multidemensional array by owner. */
            foreach ($galleries as $galleryId => $gallery) {
                $gallery_groups[$gallery->get('owner')]['galleries'][$gallery->getId()] = $gallery;
            }

            foreach ($gallery_groups as $key => $val) {
                $identity = &Identity::singleton('none', $key);
                $gallery_groups[$key]['title'] = $identity->getValue('fullname');
                if (!$gallery_groups[$key]['title']) {
                    $gallery_groups[$key]['title'] = $key;
                }
                $gallery_groups[$key]['title'] = sprintf(_("%s's Galleries"), $gallery_groups[$key]['title']);
            }
            break;

        default:
            /* Put galleries in multidimensional array. */
            $gallery_groups['list']['galleries'] = $galleries;
            $gallery_groups['list']['title'] = _("Gallery List");
        }

        /* Only sort the galleries if we're not getting an
         * entire tree. */
        if (!$allLevels) {
            /* Sort gallery groups by title. */
            uasort($gallery_groups, create_function('$a, $b', 'return strcasecmp($a[\'title\'], $b[\'title\']);'));

            /* Sort each gallery group. */
            if ($sortdown) {
                $sort_function = create_function('$a, $b', 'return strcasecmp($b->get(\'' . $sort[$sortby] . '\'), $a->get(\'' . $sort[$sortby] . '\'));');
            } else {
                $sort_function = create_function('$a, $b', 'return strcasecmp($a->get(\'' . $sort[$sortby] . '\'), $b->get(\'' . $sort[$sortby] . '\'));');
            }

            foreach ($gallery_groups as $key => $group) {
                uasort($group['galleries'], $sort_function);
                $gallery_groups[$key]['galleries'] = $group['galleries'];
            }
        }

        return $gallery_groups;
    }

    /**
     * Counts the number of current user's gallery list from storage.
     *
     * @param integer $perm        (optional) The level of permissions to
     *                             require for a gallery to return it.
     * @param string  $owner       (optional) Only count galleries that this user owns.
     * @param string  $parent      (optional) The parent share to start searching at.
     * @param boolean $allLevels   (optional) Return all levels, or just the direct children
     *                             of $parent? Defaults to all levels.
     *
     * @return integer  Returns the number of matching, sorted galleries.
     */
    function countGalleries($perm = PERMS_SHOW,
                            $owner = null,
                            $parent = DATATREE_ROOT,
                            $allLevels = true)
    {
        global $ansel_shares, $notification;

        $galleries = $ansel_shares->countShares(Auth::getAuth(), $perm, $owner, $parent, $allLevels);
        if (is_a($galleries, 'PEAR_Error')) {
            $notification->push(sprintf(_("An error occurred listing galleries: %s"), $galleries->getMessage()), 'horde.error');
            return 0;
        } else {
            return $galleries;
        }
    }

    /**
     * Return a string containing an <option> listing of the given
     * gallery array.
     *
     * @param array $galleries  The gallery array to generate a
     *                          list from.  Must be in the format
     *                          returned by listGalleries().
     *
     * @param string $galleryId If specified, this will be the
     *                          gallery that is selected by
     *                          default in the returned option
     *                          list.
     *
     * @return string  The <option> list.
     */
    function selectGalleries($galleries, $galleryId = '')
    {
        $gallerylist = '';
        foreach ($galleries as $groupId => $group) {
            foreach ($group['galleries'] as $id => $gallery) {
                $selected = ($galleryId == $id) ? ' selected="selected"' : '';
                $indent = substr_count($id, ':');
                $gallerylist .= '<option value="' . $gallery->getId() . '"' . $selected . '>' . str_repeat('&nbsp;', 2 * $indent) . htmlspecialchars($gallery->get('name')) . '</option>';
            }
        }
        return $gallerylist;
    }

    /**
     * Create and initialise a new gallery object.
     *
     * @access public
     *
     * @param string       $galleryId   The gallery id.
     * @param array        $attributes  The gallery attributes
     * @param object Perms $perm        (optional) The permissions for the gallery if
     *                                  the defaults are not desirable.
     *
     * @return object DataTreeObject_Gallery  a new gallery object or PEAR_Error.
     */
    function &createGallery($galleryId = '', $attributes = array(), $perm = null)
    {
        global $ansel_shares;

        if ($ansel_shares->exists($galleryId)) {
            $error = sprintf(_("Gallery Unique name '%s' already exists."), $galleryId);
            Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
            return PEAR::raiseError($error);
        }

        $gallery = &$ansel_shares->newGallery($galleryId);
        if (is_a($gallery, 'PEAR_Error')) {
            return $gallery;
        }

        // Required values.
        if (empty($attributes['owner'])) {
            $attributes['owner'] = Auth::getAuth();
        }
        if (empty($attributes['name'])) {
            $attributes['name'] = _("Unnamed");
        }
        if (empty($attributes['desc'])) {
            $attributes['desc'] = '';
        }

        foreach ($attributes as $attribute => $value) {
            $gallery->set($attribute, $value);
        }

        $id = $ansel_shares->addGallery($gallery);
        if (is_a($id, 'PEAR_Error')) {
            $error = sprintf(_("The gallery \"%s\" could not be created: %s"), $attributes['name'], $id->getMessage());
            Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
            return PEAR::raiseError($error);
        }

        // Set initial permissions.
        if (empty($perm)) {
            $perm = &$gallery->getPermission();
            $perm->addGuestPermission(PERMS_SHOW | PERMS_READ, false);
            $perm->addDefaultPermission(PERMS_SHOW | PERMS_READ, false);
        }

        $gallery = &$ansel_shares->getGallery($id);
        $gallery->setPermission($perm);
        $gallery->save();

        return $gallery;
    }

    /**
     * Return a link to an image, suitable for use in an <img/> tag
     * (or a Horde::img() call, with the path parameter set to
     * ''). Takes into account $conf['vfs']['direct'] and other
     * factors.
     *
     * @param string $imageId    The id of the image.
     * @param string $view       (optional) The view ('screen', 'thumb', or 'full')
     *                           to show. Defaults to screen.
     * @param boolean $full      (optional) Return a path that includes the server name?
     *
     * @return string  The image path.
     */
    function getImageUrl($imageId, $view = 'screen', $full = false)
    {
        global $conf, $ansel_shares, $registry;

        if (empty($imageId) || $imageId == DATATREE_ROOT) {
            return $registry->getImageDir() . '/error.png';
        } elseif ($conf['vfs']['direct']) {
            // We have to make sure the image exists first, since we
            // won't be going through image.php to auto-create it.
            $image = &$ansel_shares->getImage($imageId);
            if (is_a($image, 'PEAR_Error')) {
                return false;
            }
            if (is_a($result = $image->createView($view), 'PEAR_Error')) {
                return $result;
            }

            $path = substr($image->getId(), -2) . '/' . $view;
            if ($full) {
                return Horde::url('/ansel-images/' . $path . '/' . rawurlencode($image->getVFSName($view)), true, -1);
            } else {
                return '/ansel-images/' . $path . '/' . rawurlencode($image->getVFSName($view));
            }
        } else {
            $image = &$ansel_shares->getImage($imageId);
            if (is_a($image, 'PEAR_Error')) {
                return false;
            }
            $url = Util::addParameter('image.php', array('gallery' => $image->get('gallery'),
                                                         'image' => $image->getId(),
                                                         'actionID' => 'image' . $view));
            return Horde::applicationUrl($url, $full);
        }
    }

    function &getImageObject()
    {
        global $conf;
        require_once 'Horde/Image.php';
        $params = array('type' => $conf['image']['type'], 'temp' => Horde::getTempDir());
        if (!empty($conf['image']['convert'])) {
            return Horde_Image::factory('im', $params);
        } else {
            return Horde_Image::factory('gd', $params);
        }
    }

    /**
     * Read an image from the filesystem.
     *
     * @access public
     *
     * @param string         $file      The filename of the image.
     * @param optional array $override  Overwrite the file array with these values.
     *
     * @return array $image  the image data of the file as an array or PEAR_Error
     */
    function &getImageFromFile($file, $override = array())
    {
        if (!file_exists($file)) {
            return PEAR::raiseError(sprintf(_("The file '%s' doesn't exist."), $file));
        }

        // Get the mime type of the file (and make sure it's an
        // image).
        require_once 'Horde/MIME/Magic.php';
        $mime_type = MIME_Magic::analyzeFile($file);
        if (strpos($mime_type, 'image') === false) {
            return PEAR::raiseError(sprintf(_("Can't get unknown file type '%s'."), $file));
        }

        $image = array('filename' => basename($file),
                       'description' => '',
                       'data' => file_get_contents($file),
                       'default' => 0,
                       'type' => $mime_type
                       );

        // Override the array, for example if we're setting the
        // filename to something else.
        if (count($override)) {
            $image = array_merge($image, $override);
        }

        return $image;
    }

    /**
     * Check to see if a particular image manipulation function is
     * available.
     *
     * @param string $feature  The name of the function.
     *
     * @return boolean  True if the function is available.
     */
    function isAvailable($feature)
    {
        static $capabilities;

        if (!isset($capabilities)) {
            $im = &Ansel::getImageObject();
            $capabilities = $im->getCapabilities();
        }

        return in_array($feature, $capabilities);
    }

    /**
     * Calculates the number of days since the given timestamp.
     *
     * @param integer $date  Calculate the number of days since
     *                       this timestamp.
     *
     * @return string  Human-formatted days since $date.
     */
    function daysSince($date)
    {
        $days = round((time() - $date) / 86400);

        if ($days == 0) {
            $since = _("Today");
        } elseif ($days == 1) {
            $since = _("Yesterday");
        } else {
            $since = sprintf("%s days ago", $days);
        }

        return $since .= strftime(' (%B %e, %Y)', $date);
    }

    /**
     * Build Ansel's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry, $browser, $print_link, $ansel_shares;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        $menu->add(Horde::applicationUrl('list.php'), _("_Galleries"), 'mypics.png', null, null, null, (basename($_SERVER['PHP_SELF']) == 'index.php' || Util::getFormData('owner', false) != Auth::getAuth()) ? 'current' : '__noselection');
        if (Auth::getAuth()) {
            $menu->add(Horde::applicationUrl(Util::addParameter('list.php', 'owner', Auth::getAuth())), _("My Galleries"), 'mygalleries.png', null, null, null, (Util::getFormData('owner', false) == Auth::getAuth()) ? 'current' : '__noselection');
        }

        /* If we have a gallery ID, include links to get back to that
         * gallery, its slideshow, and zipfile download. */
        if (Util::getFormData('gallery')) {
            $gallery = $ansel_shares->getGallery(Util::getFormData('gallery'));
            if (!is_a($gallery, 'PEAR_Error')) {
                $menu->add(Horde::applicationUrl(Util::addParameter('view.php',
                                                                    array('gallery' => $gallery->getId(),
                                                                          'page' => Util::getFormData('page')))),
                           $gallery->get('name'), 'ansel.png');

                if ($gallery->get('num-images')) {
                    $menu->add(Horde::applicationUrl(Util::addParameter('slideshow.php', 'gallery', $gallery->getId())), _("Slide Show"), 'slideshow.png');

                    if (Auth::getAuth()) {
                        $galleryurl = Util::addParameter('gallery.php', 'gallery', $gallery->getId());
                        $menu->add(Horde::applicationUrl(Util::addParameter($galleryurl, 'actionID', 'downloadzip')), _("Download"), 'mime/compressed.png', $registry->getImageDir('horde'), null, null, '__noselection');
                    }
                }
            }
        }

        /* Let authenticated users create new galleries. */
        if (Auth::isAdmin() ||
            !$GLOBALS['perms']->exists('ansel') ||
            $GLOBALS['perms']->hasPermission('ansel', Auth::getAuth(), PERMS_EDIT)) {
            $menu->add(Horde::applicationUrl(Util::addParameter('gallery.php', 'actionID', 'add')), _("_New Gallery"), 'add.png');
        }

        /* Print. */
        if ($conf['menu']['print'] && isset($print_link) && $browser->hasFeature('javascript')) {
            $menu->add("javascript:popup('$print_link'); return false;", _("Print"), 'print.png', $registry->getImageDir('horde'));
        }

        /* The search menu item. */
        $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
