#!/usr/bin/php -q
<?php
/**
* $Horde: ansel/scripts/ansel.php,v 1.8 2004/11/13 04:11:10 chuck Exp $
*
* This script interfaces with Ansel via the command-line
*
* See the enclosed file COPYING for license information (GPL). If you
* did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
*
* @author Vijay Mahrra <webmaster@stain.net>
*/

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../..');
@define('ANSEL_BASE', HORDE_BASE . '/ansel');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment.
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Share.php';
require_once 'Console/Getopt.php';
Horde_CLI::init();
$cli = &Horde_CLI::singleton();
$registry = &Registry::singleton();
$registry->pushApp('ansel', false);
$shares = &Horde_Share::singleton($registry->getApp());

// We accept the user name on the command-line.
$ret = Console_Getopt::getopt(Console_Getopt::readPHPArgv(), 'hu:p:lc:g:a:d:',
                              array('help', 'username=', 'password=', 'list', 'create=', 'gallery=', 'add=', 'dir='));

if (is_a($ret, 'PEAR_Error')) {
    $error = _("Couldn't read command-line options.");
    Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_DEBUG);
    $cli->fatal($error);
}

list($opts, $args) = $ret;

// Show help and exit if no arguments were set.
if (!count($opts)) {
    showHelp();
    exit;
}

foreach ($opts as $opt) {
    list($optName, $optValue) = $opt;
    switch ($optName) {
    case 'u':
    case '--username':
        $username = $optValue;
        break;

    case 'p':
    case '--password':
        $password = $optValue;
        break;

    case 'l':
    case '--list':
        $list = true;
        break;

    case 'c':
    case '--create':
        list($createGalleryId, $gallery_name, $gallery_desc) = split('/', $optValue, 3);
        break;

    case 'g':
    case '--gallery':
        $galleryId = $optValue;
        break;

    case 'a':
    case '--add':
        $file = $optValue;
        break;

    case 'd':
    case '--dir':
        $dir = $optValue;
        break;

    case 'h':
    case '--help':
        showHelp();
        exit;
    }
}

// Login to horde if username & password are set.
if (!empty($username) && !empty($password)) {
    $auth = &Auth::singleton($conf['auth']['driver']);
    if (!$auth->authenticate($username, array('password' => $password))) {
        $error = _("Login is incorrect.");
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
        $cli->fatal($error);
    } else {
        $msg = sprintf(_("Logged in successfully as '%s'."), $username);
        Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $cli->message($msg, 'cli.success');
    }
}

// Create a gallery.
if (!empty($createGalleryId)) {
    $gallery = &Ansel::createGallery($createGalleryId, array('name' => $gallery_name,
                                                             'desc' => $gallery_desc));
    if (is_a($gallery, 'PEAR_Error')) {
        $galleryId = null;
        $error = sprintf(_("The gallery \"%s\" couldn't be created: %s"), $gallery_name, $gallery->getMessage());
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
        $cli->fatal($error);
    } else {
        $msg = sprintf(_("The gallery \"%s\" was created successfully."), $createGalleryId);
        Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $cli->message($msg, 'cli.success');
    }
}

// List galleries.
if (!empty($list)) {
    $galleries = Ansel::listGalleries();
    if (is_a($galleries, 'PEAR_Error')) {
        $error = _("Couldn't list galleries.");
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $cli->fatal($error);
    }
    $cli->message(_("Listing Gallery/Name"), 'cli.success');
    foreach ($galleries as $groupId => $group) {
        foreach ($group['galleries'] as $id => $gallery) {
            $name = $gallery->get('name');
            $msg = "$id/$name";
            $cli->writeln($msg);
            Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        }
    }
}

// Choose the gallery to add to (or use the created one).
if (!empty($galleryId)) {
    if (!$galleryId || !$ansel_shares->exists($galleryId)) {
        $error = sprintf(_("Invalid gallery '%s' specified."), $galleryId);
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_WARNING);
        $cli->fatal($error);
    } else {
        $gallery = &$ansel_shares->getShare($galleryId);
        if (is_a($gallery, 'PEAR_ERROR') || !$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
            $error = sprintf(_("You cannot add photos to the gallery '%s'."), $galleryId);
            Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_WARNING);
            $cli->fatal($error);
        }
    }
}

// Add an image from the filesystem.
if (!empty($file) && isset($gallery) && !is_a($gallery, 'PEAR_Error')) {
    $image = &Ansel::getImageFromFile($file);
    if (is_a($image, 'PEAR_Error')) {
        Horde::logMessage($image->getMessage(), __FILE__, __LINE__, PEAR_LOG_WARNING);
        $cli->fatal($image->getMessage());
    }

    $cli->message(sprintf(_("Storing image '%s'..."), $file), 'cli.message');
    $image_id = $gallery->addImage($image);
    if (is_a($image_id, 'PEAR_Error')) {
        $error = sprintf(_("There was a problem adding the image '%s' to gallery '%s'."), basename($file), $galleryId);
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
        $cli->fatal($error);
    }
    $msg = sprintf(_("Successfully added image '%s' to gallery '%s'."), basename($file), $galleryId);
    $cli->message($msg, 'cli.success');
    Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_NOTICE);
}

// Add all images from a directory on the filesystem.
if (!empty($dir) && isset($gallery) && !is_a($gallery, 'PEAR_Error')) {
    $msg = addDirToGallery($dir, $gallery);
    if (is_a($msg, 'PEAR_Error')) {
        Horde::logMessage($msg->getMessage(), __FILE__, __LINE__, PEAR_LOG_ERR);
        $cli->fatal($msg->getMessage());
    }
    if (count($msg) > 0) {
        $msg = sprintf(_("Successfully added %d image(s) (%s) to gallery '%s' from '%s'."),
                        count($msg), join(', ', $msg), $galleryId, $dir);
        $cli->message($msg, 'cli.success');
        Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_NOTICE);
    } else {
        $msg = sprintf(_("The directory '%s' had no valid images."), $dir);
        $cli->message($msg, 'cli.warning');
        Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_WARNING);
    }
}

exit;


/**
 * Read an all images from a directory into the currently selected gallery
 * @param string $dir
 * @param object $gallery &$ansel_shares->getShare($galleryId)
 * @return array $added_images an array of all the images that were successfully added to the gallery
 * @return array $image the image data of the file as an array or PEAR_Error
 */
function addDirToGallery($dir = '', &$gallery)
{
    global $cli, $galleryId, $ansel_shares;

    if (!file_exists($dir)) {
        $error = sprintf(_("The directory '%s' doesn't exist."), $dir);
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
        return PEAR::raiseError($error);
    }

    // Read all the files into an array.
    $files_array = array();
    $h = opendir($dir);
    while (false !== ($file = readdir($h))) {
        if ($file == '.' || $file == '..') {
            continue;
        }
        array_push($files_array, $file);
    }
    closedir($h);

    if (!count($files_array)) {
        $error = sprintf(_("The directory '%s' is empty."), $dir);
        Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
        return PEAR::raiseError($error);
    }
    chdir($dir);

    // Process each file and upload to the gallery.
    $added_images = array();
    foreach ($files_array as $file) {
        $image = Ansel::getImageFromFile($dir . '/' . $file);
        if (is_a($image, 'PEAR_Error')) {
            Horde::logMessage($image->getMessage(), __FILE__, __LINE__, PEAR_LOG_WARNING);
            $cli->message($image->getMessage(), 'cli.error');
            continue;
        }

        $cli->message(sprintf(_("Storing image '%s'..."), $file), 'cli.message');
        $image_id = $gallery->addImage($image);
        if (is_a($image_id, 'PEAR_Error')) {
            $error = sprintf(_("There was a problem adding the image '%s' to gallery '%s'."), $file, $galleryId);
            Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
            $cli->message($image_id->getMessage(), 'cli.error');
            continue;
        }

        array_push($added_images, $file);
    }

    return $added_images;
}

/**
 * Show the command line arguments that the script accepts.
 */
function showHelp()
{
    global $cli;

    $cli->writeln(sprintf(_("Usage: %s [OPTIONS]..."), basename(__FILE__)));
    $cli->writeln();
    $cli->writeln(_("Mandatory arguments to long options are mandatory for short options too."));
    $cli->writeln();
    $cli->writeln(_("-h, --help                   Show this help"));
    $cli->writeln(_("-l, --list                   List galleries"));
    $cli->writeln(_("-c, --create[=shortname/name/description]  Create gallery (and use it)"));
    $cli->writeln(_("-g, --gallery[=shortname]    Select gallery to use"));
    $cli->writeln(_("-a, --add[=filename]         Add local file to selected gallery"));
    $cli->writeln(_("-d, --dir[=directory]        Add all files from the directory to the selected\n                             gallery"));
    $cli->writeln(_("-u, --username[=username]    Horde login username"));
    $cli->writeln(_("-p, --password[=password]    Horde login password"));
    $cli->writeln();
}
